! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
! Module: binned_att
!
! Authors:  Ian Stewart - SSC/LUX
!          
! Description: This module contains routines associated with binned-up attitude data.
! 
! Comments: 
!
! $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
!- 
module binned_att

! You should write the module name in the 'use' statement, the package name
! (and version number) in the DEPEND file, and the '*' from the lib*.a name
! in <package>/src/Makefile.

  use dal
  use errorHandling
  use caloaldefs	! for SpacecraftAttitudeType
  use caltypes		! for IntervalsT
  use oal

  use array_utils	! for getBinNumber
  use constants
  use dal_aux
  use intervals_aux	! for gtiVecType, andIntervals.

  implicit none 

  private 

  public initializeBinnedAtt
  public binUpAttitudeFromOdf
  public sampleBinnedAttitude
!  public binUpAttitude
  public writeBinnedAttitude
  public readBinnedAttitude
  public gtiWeightBinnedAttitude
!  public calcBinnedAttGtis	! Abolished. It's horrible.
!  public openGtiWeightBinnedAtt	! ditto
  public constructDummyBinnedAtt

  ! Since the first bite at the cherry, I've thought some more about the concepts I want to encode in the binned-attitude formalism. The problem with the first incarnation was that attBin data became mixed up with GTI data via the endsUnbounded flags and keywords. The only way I can think to unsnarl this is to assume that the ends are always unbounded. Then, if one wants for example to perform instrument-to-sky coordinate transforms, weighting by duration in each attitude bin, one is now obliged to get hold of some sort of GTI data and use this to calculate the weights; without GTI data, weights for the first and last attitude will be meaningless.
  !
  ! The attBinType definition has been changed as follows:
  !	- binEndTime changed to binStartTime, with the convention understood that the value for the first attitude is ignored;
  !	- fracExp has been changed to gtiDuration, with its type changed from single to double.
  !
  ! The table format for n attitudes now comprises:
  !	- Columns with n-1 rows:
  !		- 'TSTART',  REAL64 
  !		- 'RA',      REAL64 
  !		- 'DEC',     REAL64 
  !		- 'PA',      REAL64 
  !		- 'IS_GOOD', BOOLEAN
  ! Plus optional column:
  !		- 'WEIGHTS', REAL64
  !
  !	- Table keywords:
  !		- RA_FIRST
  !		- DECFIRST
  !		- PA_FIRST
  !		- FIRST_OK
  !		- WGTFIRST
  !		- RA_PNT
  !		- DEC_PNT
  !		- PA_PNT
  !		- RA_DELTA
  !		- DECDELTA
  !		- PA_DELTA
  !		- AVRG_PNT
  !

  type, public :: attBinType
    type(SpacecraftAttitudeType) ::&
      att
    real(double) ::&
      binStartTime=0.0d0,&
      gtiDuration=0.0d0
    logical ::&
      isGood=.false.
  end type attBinType

contains 

  !---------------------------------------------------------------------
  subroutine initializeBinnedAtt(binnedAtt)
  !---------------------------------------------------------------------

    implicit none

    type(attBinType), intent(out) :: binnedAtt(:)

    !-------------------------------------------------------------------
    ! Start

    binnedAtt%att%ra       = 0.0d0
    binnedAtt%att%dec      = 0.0d0
    binnedAtt%att%apos     = 0.0d0
    binnedAtt%binStartTime = 0.0d0
    binnedAtt%gtiDuration  = 0.0d0
    binnedAtt%isGood       = .false.

  end subroutine initializeBinnedAtt

  !---------------------------------------------------------------------
  subroutine binUpAttitudeFromOdf(time, maxDelta, attInBin, attIsGood&
  , attIsNew)
  !---------------------------------------------------------------------
  ! The subroutine samples the attitude from the AHF in the ODF. If the
  ! attitude is good, the returned value is the last good attitude unless
  ! (i) this is the first time a good attitude has been returned from the
  ! AHF, or (ii) the difference between the present attitude and the last
  ! good one exceeds certain limits; in either of these cases, the
  ! attitude returned by the routine is that just obtained from the AHF.
  !
  ! NOTE! The OAL should be set up for attitudeFromAhf before this sub
  ! is called.

    implicit none 

    real(double),                 intent(in)  :: time
    type(SpacecraftAttitudeType), intent(in)  :: maxDelta
    type(SpacecraftAttitudeType), intent(out) :: attInBin
    logical,                      intent(out) :: attIsGood, attIsNew
    ! Even if (.not. attIsGood), attIsNew will be .true. if (lastAttWasGood). 

    type(SpacecraftAttitudeType) :: att
    logical, save :: noGoodAttsYet = .true., lastAttWasGood = .false.
    type(SpacecraftAttitudeType), save :: baselineAtt
    logical :: raIsNew, decIsNew, paIsNew

    !-------------------------------------------------------------------
    ! Start:

    attInBin%ra   = 0.0d0
    attInBin%dec  = 0.0d0
    attInBin%apos = 0.0d0

    if (OAL_getAttitude(time, att)) then 
      attIsGood = .true.
    else
      attIsGood = .false.
    end if

!attIsGood = .true.
!att%ra = 1.5707963268d0
!att%dec = 0.0d0
!att%apos = 0.0d0
!print *, 'CAREFUL! Temporary test routine activated.'

    attIsNew = .false. ! default

    if (attIsGood) then
      if (noGoodAttsYet) then
        ! This is the FIRST good att bin, ie. baselineAtt is NOT yet set, so is set.
        attInBin = att
        baselineAtt = att	! Revised and "saved".

      else ! some good atts have already been found, thus guaranteeing
      ! the existence of a baseline to test against.
      ! Namely, a bin may include time spans that are bad (in terms of att),
      ! where attIsNew (= .true) will be returned in that case.
        raIsNew  = .false.
        decIsNew = .false.
        paIsNew  = .false.

        if (abs(att%ra - baselineAtt%ra) > maxDelta%ra) then
!***** NOTE! At present this does NOT cater for situations in which the attitude wanders across ra=0/pi, dec=+-90 or apos=0/pi. Better would be to work in a pnt-centric coordinate system.

          baselineAtt%ra = att%ra	! Duplicated(?) as will be overwritten?
          raIsNew = .true.
        endif
        if (abs(att%dec - baselineAtt%dec) > maxDelta%dec) then 
          baselineAtt%dec = att%dec	! Duplicated(?) as will be overwritten?
          decIsNew = .true.
        endif
        if (abs(att%apos - baselineAtt%apos) > maxDelta%apos) then 
          baselineAtt%apos = att%apos	! Duplicated(?) as will be overwritten?
          paIsNew = .true.
        endif

        if (raIsNew .or. decIsNew .or. paIsNew) then 
          attInBin = att
          baselineAtt = att		! Set here.
          attIsNew = .true.
        else
          attInBin = baselineAtt	! Set current att to be the same as the "saved".
        end if
      end if

      if (.not. lastAttWasGood) attIsNew = .true.
      noGoodAttsYet = .false.
      lastAttWasGood = .true.

    else ! att is not good
      if (lastAttWasGood) then
        attIsNew = .true.
      else ! last att was no good, either.
      end if
      lastAttWasGood = .false.
    end if

  end subroutine binUpAttitudeFromOdf


  !---------------------------------------------------------------------
  subroutine sampleBinnedAttitude(binnedAtt, time, outAtt, attIsGood&
  , attIsNew)
  !---------------------------------------------------------------------

    implicit none

    type(attBinType),             intent(in)  :: binnedAtt(:)
    real(double),                 intent(in)  :: time
    type(SpacecraftAttitudeType), intent(out) :: outAtt
    logical,                      intent(out) :: attIsGood, attIsNew

    integer :: numAttBins, binNum
    logical, save :: noGoodAttsYet = .true.
    integer, save :: binOfLastGoodAtt = 0

    !-------------------------------------------------------------------
    ! Start

    outAtt%ra   = 0.0d0
    outAtt%dec  = 0.0d0
    outAtt%apos = 0.0d0
    attIsGood = .false.
    attIsNew  = .false.

    numAttBins = size(binnedAtt)

    ! Problem in accessing element two of the binned attitude as this
    ! may not exist if the attitude file had no entries which is possible.
    ! Use the first attitude bin if there is only one or if the time is
    ! before the start of the second bin, otherwise select the right bin
    binNum = 1
    if (numAttBins > 1) then
       if (time < binnedAtt(2)%binStartTime) then
          binNum = 1
       else if (time >= binnedAtt(numAttBins)%binStartTime) then
          binNum = numAttBins
       else
          binNum = 1 + getBinNumber(binnedAtt(2:)%binStartTime, time) ! in ssclib/array_utils.
       endif
    endif

    if (binnedAtt(binNum)%isGood) then
       attIsGood = .true.
       outAtt = binnedAtt(binNum)%att
    else
       attIsGood = .false.
    end if

    if (attIsGood) then
      if (noGoodAttsYet) then
        noGoodAttsYet = .false.
        binOfLastGoodAtt = binNum
        attIsNew = .true.

      else ! we can be sure that binOfLastGoodAtt is defined.
        if (binNum == binOfLastGoodAtt) then
          attIsNew = .false.
        else
          binOfLastGoodAtt = binNum
          attIsNew = .true.
        end if
      end if
    end if

  end subroutine sampleBinnedAttitude

  !---------------------------------------------------------------------
  subroutine binUpAttitude()
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

!*****

  end subroutine binUpAttitude

  !---------------------------------------------------------------------
  subroutine writeBinnedAttitude(binnedAttSetName, binnedAttTabName&
  , binnedAtt, pntDeg, maxDelta, pntStyle, writeWeightColumn)
  !---------------------------------------------------------------------

    implicit none

    character(*),     intent(in)           :: binnedAttSetName, binnedAttTabName
    type(attBinType), intent(in)           :: binnedAtt(:)
    type(SpacecraftAttitudeType),&
                      intent(in)           :: pntDeg, maxDelta
    character(*),     intent(in)           :: pntStyle
    logical,          intent(in), optional :: writeWeightColumn

    logical :: writeWeights
    type(DataSetT) :: binnedAttSet
    type(TableT) :: binnedAttTab
    real(double), pointer :: time(:), raDeg(:), decDeg(:), paDeg(:), weights(:)
    logical(bool), pointer :: isGood(:)

    !-------------------------------------------------------------------
    ! Start

    writeWeights = .false. ! default
    if (present(writeWeightColumn)) then
      if (writeWeightColumn) writeWeights = .true.
    end if

    binnedAttSet = dataSet(binnedAttSetName, CREATE)
    binnedAttTab = addTable(binnedAttSet, binnedAttTabName, size(binnedAtt)-1)

    call setAttribute(binnedAttTab, 'RA_FIRST', binnedAtt(1)%att%ra   / PiOn180, "deg")
    call setAttribute(binnedAttTab, 'DECFIRST', binnedAtt(1)%att%dec  / PiOn180, "deg")
    call setAttribute(binnedAttTab, 'PA_FIRST', binnedAtt(1)%att%apos / PiOn180, "deg")
    call setAttribute(binnedAttTab, 'FIRST_OK', binnedAtt(1)%isGood)
    if (writeWeights) then
      call setAttribute(binnedAttTab, 'WGTFIRST', binnedAtt(1)%gtiDuration)
    else
      call setAttribute(binnedAttTab, 'WGTFIRST', 0.0d0)
    end if
    call setAttribute(binnedAttTab, 'RA_PNT',   pntDeg%ra,"deg"&
      ,"Average right ascension of the s/c pointing axis")
    call setAttribute(binnedAttTab, 'DEC_PNT',  pntDeg%dec,"deg"&
      ,"Average declination of the s/c pointing axis")
    call setAttribute(binnedAttTab, 'PA_PNT',   pntDeg%apos,"deg"&
      ,"Average position angle of the s/c pointing axis")
    call setAttribute(binnedAttTab, 'RA_DELTA', maxDelta%ra   / PiOn180)
    call setAttribute(binnedAttTab, 'DECDELTA', maxDelta%dec  / PiOn180)
    call setAttribute(binnedAttTab, 'PA_DELTA', maxDelta%apos / PiOn180)
    call setAttribute(binnedAttTab, 'AVRG_PNT', trim(pntStyle)&
      ,"Meaning of _PNT values (mean, median or fixed)")

    time   => real64Data(addColumn(binnedAttTab, 'TSTART',  REAL64, 's'))
    raDeg  => real64Data(addColumn(binnedAttTab, 'RA',      REAL64, 'deg'))
    decDeg => real64Data(addColumn(binnedAttTab, 'DEC',     REAL64, 'deg'))
    paDeg  => real64Data(addColumn(binnedAttTab, 'PA',      REAL64, 'deg'))
    isGood => boolData(  addColumn(binnedAttTab, 'IS_GOOD', BOOLEAN))

    time   = binnedAtt(2:)%binStartTime
    raDeg  = binnedAtt(2:)%att%ra   / PiOn180
    decDeg = binnedAtt(2:)%att%dec  / PiOn180
    paDeg  = binnedAtt(2:)%att%apos / PiOn180
    isGood = binnedAtt(2:)%isGood

    if (writeWeights) then
      weights => real64Data(addColumn(binnedAttTab, 'WEIGHTS',  REAL64))
      weights = binnedAtt(2:)%gtiDuration
    end if

    call release(binnedAttSet)

!****** also write HDU and other OGIP keywords????

  end subroutine writeBinnedAttitude

  !---------------------------------------------------------------------
  subroutine readBinnedAttitude(binnedAttSetName, binnedAttTabName&
  , binnedAtt, noWeightColumn)
  !---------------------------------------------------------------------
  ! Note! The calling routine should deallocate the pointer binnedAtt after use.

    implicit none

    character(*),     intent(in)            :: binnedAttSetName, binnedAttTabName
    type(attBinType), pointer               :: binnedAtt(:)
    logical,          intent(out), optional :: noWeightColumn

    type(DataSetT) :: binnedAttSet
    type(TableT) :: binnedAttTab
    integer :: numAttBins
    logical(bool), allocatable :: attIsGoodBool(:)

    !-------------------------------------------------------------------
    ! Start:

    binnedAttSet = dataset(binnedAttSetName, READ)
    binnedAttTab = table(binnedAttSet, binnedAttTabName)

    numAttBins = 1 + numberOfRows(binnedAttTab)
    allocate(binnedAtt(numAttBins))
    allocate(attIsGoodBool(numAttBins))

    call initializeBinnedAtt(binnedAtt)

    binnedAtt(1)%att%ra   = real64Attribute(binnedAttTab,  'RA_FIRST')
    binnedAtt(1)%att%dec  = real64Attribute(binnedAttTab,  'DECFIRST')
    binnedAtt(1)%att%apos = real64Attribute(binnedAttTab,  'PA_FIRST')
    binnedAtt(1)%isGood   = booleanAttribute(binnedAttTab, 'FIRST_OK')

    call readColDataToFixed(binnedAttTab, 'TSTART',  binnedAtt(2:)%binStartTime)
    call readColDataToFixed(binnedAttTab, 'RA',      binnedAtt(2:)%att%ra)
    call readColDataToFixed(binnedAttTab, 'DEC',     binnedAtt(2:)%att%dec)
    call readColDataToFixed(binnedAttTab, 'PA',      binnedAtt(2:)%att%apos)
    call readColDataToFixed(binnedAttTab, 'IS_GOOD', attIsGoodBool(2:))
    ! These calls are in ssclib/dal_aux.

    if (hasColumn(binnedAttTab, 'WEIGHTS')) then
      call readColDataToFixed(binnedAttTab, 'WEIGHTS',  binnedAtt(2:)%gtiDuration)
      binnedAtt(1)%gtiDuration = real64Attribute(binnedAttTab, 'WGTFIRST')
      if (present(noWeightColumn)) noWeightColumn = .false.
    else
      if (present(noWeightColumn)) noWeightColumn = .true.
    end if

    binnedAtt%att%ra   = binnedAtt%att%ra   * PiOn180
    binnedAtt%att%dec  = binnedAtt%att%dec  * PiOn180
    binnedAtt%att%apos = binnedAtt%att%apos * PiOn180
    binnedAtt(2:)%isGood = attIsGoodBool(2:)

    call release(binnedAttSet)

  end subroutine readBinnedAttitude

  !---------------------------------------------------------------------
  subroutine gtiWeightBinnedAttitude(gtis, binnedAtt)
  !---------------------------------------------------------------------

    implicit none

    type(IntervalT),  intent(in)    :: gtis(:)
    type(attBinType), intent(inout) :: binnedAtt(:)

    integer :: numAttBins, numGtis, attBinNum
    type(IntervalT), allocatable :: attIntervals(:)
    type(IntervalT), pointer :: andedIntervals(:)

    !-------------------------------------------------------------------
    ! Start

    numAttBins = size(binnedAtt)
  if (numAttBins <= 0) return

    numGtis = size(gtis)
    if (numGtis <= 0) then
      binnedAtt%gtiDuration = 0.0
  return
    end if

    if (numAttBins == 1) then
      binnedAtt(1)%gtiDuration = sum(gtis%upper) - sum(gtis%lower)
  return
    end if

    allocate(attIntervals(numAttBins))

    attIntervals(1)%lower = min(gtis(1)%lower, binnedAtt(2)%binStartTime)
    attIntervals(1)%upper = binnedAtt(2)%binStartTime

    do attBinNum = 2, numAttBins - 1
      attIntervals(attBinNum)%lower = attIntervals(attBinNum-1)%upper
      attIntervals(attBinNum)%upper = binnedAtt(attBinNum+1)%binStartTime
    end do

    attIntervals(numAttBins)%lower = attIntervals(numAttBins-1)%upper
    attIntervals(numAttBins)%upper = max(gtis(numGtis)%upper, binnedAtt(numAttBins)%binStartTime)

    do attBinNum = 1, numAttBins
      call andIntervals(attIntervals(attBinNum), gtis, andedIntervals)
      ! in ssclib/intervals_aux

      binnedAtt(attBinNum)%gtiDuration = sum(andedIntervals%upper)&
         - sum(andedIntervals%lower)

      deallocate(andedIntervals)
    end do

  end subroutine gtiWeightBinnedAttitude

!  !---------------------------------------------------------------------
!  subroutine calcBinnedAttGtis(binnedAtt, startTimeFirstBin&
!    , endsUnbounded, gtis, gtiType, binnedAttWeights)
!  !---------------------------------------------------------------------
!
!    implicit none
!
!    type(attBinType), intent(in)  :: binnedAtt(:)
!    real(double),     intent(in)  :: startTimeFirstBin
!    logical,          intent(in)  :: endsUnbounded(2)
!    type(IntervalT),  intent(in)  :: gtis(:)
!    integer,          intent(in)  :: gtiType
!    real(single),     intent(out) :: binnedAttWeights(size(binnedAtt))
!
!    integer :: numAttBins, numGtis, attBinNum
!    type(IntervalT), allocatable :: attIntervals(:)
!    type(IntervalT), pointer :: andedIntervals(:)
!
!!    integer, parameter :: OPEN_GTI=0, CLOSED_GTI=1, GTI_IN_LIST=2
!!*** Yuk. Better to move the whole gtiVecType to here (or to intervals_aux).
!
!    !-------------------------------------------------------------------
!    ! Start
!
!    numAttBins = size(binnedAtt)
!  if (numAttBins <= 0) return
!
!    numGtis = size(gtis)
!    if (gtiType == GTI_IN_LIST .and. numGtis <= 0) then
!      binnedAttWeights = 0.0
!  return
!    end if
!
!    if (gtiType == CLOSED_GTI) then
!      binnedAttWeights = 0.0
!  return
!    end if
!
!    if (any(endsUnbounded) .and. gtiType == OPEN_GTI)&
!call error('',"")
!
!    allocate(attIntervals(numAttBins))
!    attIntervals(1)%lower = startTimeFirstBin
!    attIntervals(1)%upper = binnedAtt(1)%binEndTime
!    do attBinNum = 2, numAttBins
!      attIntervals(attBinNum)%lower = attIntervals(attBinNum-1)%upper
!      attIntervals(attBinNum)%upper = binnedAtt(attBinNum)%binEndTime
!    end do
!
!    if (endsUnbounded(1)) then
!      if (endsUnbounded(2)) then
!        attIntervals(1         )%lower = gtis(1      )%lower
!        attIntervals(numAttBins)%upper = gtis(numGtis)%upper
!
!        if (numAttBins > 1) then
!          attIntervals(1         )%lower = min(attIntervals(1         )%lower&
!            , attIntervals(1         )%upper)
!          attIntervals(numAttBins)%upper = max(attIntervals(numAttBins)%lower&
!            , attIntervals(numAttBins)%upper)
!        end if
!
!      else
!        attIntervals(1)%lower = min(gtis(1)%lower, attIntervals(1)%upper)
!      end if
!
!    else
!      if (endsUnbounded(2)) then
!        attIntervals(numAttBins)%upper = max(gtis(numGtis)%upper&
!          , attIntervals(numAttBins)%lower)
!      end if
!    end if
!
!    if (gtiType == GTI_IN_LIST) then
!      do attBinNum = 1, numAttBins
!        call andIntervals(attIntervals(attBinNum), gtis, andedIntervals)
!        ! in ssclib/intervals_aux
!
!        binnedAttWeights(attBinNum) = sum(andedIntervals%upper)&
!           - sum(andedIntervals%lower)
!
!        deallocate(andedIntervals)
!      end do
!
!    else ! gtiType == OPEN_GTI
!      binnedAttWeights(1) = binnedAtt(1)%binEndTime - startTimeFirstBin
!      do attBinNum = 2, numAttBins
!        binnedAttWeights(attBinNum) = binnedAtt(attBinNum)%binEndTime&
!          - binnedAtt(attBinNum-1)%binEndTime
!      end do
!    end if
!
!  end subroutine calcBinnedAttGtis
!
!  !---------------------------------------------------------------------
!  subroutine openGtiWeightBinnedAtt(binnedAtt, startTimeFirstBin&
!    , endsUnbounded, multiplyByGtis, binnedAttWeights)
!  !---------------------------------------------------------------------
!
!    implicit none
!
!    type(attBinType), intent(in)  :: binnedAtt(:)
!    real(double),     intent(in)  :: startTimeFirstBin
!    logical,          intent(in)  :: endsUnbounded(2)
!    logical,          intent(in)  :: multiplyByGtis
!    real(single),     intent(out) :: binnedAttWeights(size(binnedAtt))
!
!    integer :: numAttBins, attBinNum
!
!    !-------------------------------------------------------------------
!    ! Start
!
!    numAttBins = size(binnedAtt)
!  if (numAttBins <= 0) return
!
!    if (.not. multiplyByGtis) then
!      binnedAttWeights = 1.0
!  return
!    end if
!
!if (any(endsUnbounded)) call error('',"")
!
!    binnedAttWeights(1) = binnedAtt(1)%binEndTime - startTimeFirstBin
!    do attBinNum = 2, numAttBins
!      binnedAttWeights(attBinNum) = binnedAtt(attBinNum)%binEndTime&
!        - binnedAtt(attBinNum-1)%binEndTime
!    end do
!
!  end subroutine openGtiWeightBinnedAtt

  !---------------------------------------------------------------------
  subroutine constructDummyBinnedAtt(templateSetName, binnedAtt)
  !---------------------------------------------------------------------
  ! Note! The calling routine should deallocate the pointer binnedAtt after use.

    implicit none 

    character(*),     intent(in)  :: templateSetName
    type(attBinType), pointer     :: binnedAtt(:)

    type(DataSetT) :: set
    real(double) ::&
      pntRaDeg,&
      pntDecDeg,&
      pntAposDeg

    !-------------------------------------------------------------------
    ! Start:

    allocate(binnedAtt(1))
    call initializeBinnedAtt(binnedAtt)

    set = dataSet(templateSetName, READ)

    pntRaDeg    = real64Attribute(set, "RA_PNT")
    pntDecDeg   = real64Attribute(set, "DEC_PNT")
    pntAposDeg  = real64Attribute(set, "PA_PNT")

    binnedAtt(1)%att%ra   = pntRaDeg   * PiOn180
    binnedAtt(1)%att%dec  = pntDecDeg  * PiOn180
    binnedAtt(1)%att%apos = pntAposDeg * PiOn180

    binnedAtt(1)%isGood      = .true.
    binnedAtt(1)%gtiDuration = 1.0d0

    call release(set)

  end subroutine constructDummyBinnedAtt

  !---------------------------------------------------------------------
  subroutine template
  !---------------------------------------------------------------------

    implicit none

    !-------------------------------------------------------------------
    ! Start

  end subroutine template

end module binned_att
